==> matrix_sdk-0.8.2.tar.gz: lib/src/event/room/message_event.dart <==
// Copyright (C) 2019  Wilko Manger
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'package:matrix_sdk/src/event/event.dart';
import 'package:matrix_sdk/src/event/room/room_event.dart';
import 'package:matrix_sdk/src/identifier.dart';
import 'package:meta/meta.dart';

const String _messageEventType = 'm.room.message';

abstract class MessageEventContent extends EventContent {

  @override
  final String matrixType = _messageEventType;

  @protected
  String get messageType;

  EventId get inReplyToId;

  @override
  Map<String, dynamic> toJson() {
    final Map<String, dynamic> json = { 'msgtype': messageType };

    if (inReplyToId != null) {
      json.addAll({
        'm.relates_to': {
          'm.in_reply_to': {
            'event_id': inReplyToId.toString()
          }
        }
      });
    }

    return json;
  }
}

abstract class MessageEvent extends RoomEvent {

  static const String type = _messageEventType;

  @override
  final String matrixType = _messageEventType;

  @override
  final MessageEventContent content;

  MessageEvent(
    this.content,
    RoomEventArgs args)
    : super(args);

  static MessageEvent fromJson(RoomEventArgs args,
                                Map<String, dynamic> json) {

    final content = json['content'] as Map<String, dynamic>;
    final msgtype = content['msgtype'];

    var inReplyTo;
    if (content.containsKey('m.relates_to')
      && content['m.relates_to']?.containsKey('m.in_reply_to') == true) {
      final repliesTo = content['m.relates_to']['m.in_reply_to']['event_id'];
      inReplyTo = EventId(repliesTo);
    }

    switch (msgtype) {
      case ImageMessage.type:
        final body = content['body'];

        var url = content['url'];
        if (url == null) {
          return null;
        }

        var info = content['info'];
        if (info != null) {
          info = ImageInfo(
            width: content['info']['w'],
            height: content['info']['h']
          );
        }

        url = Uri.parse(url);
        return ImageMessageEvent(
          ImageMessage(
            body: body,
            url: url,
            info: info,
            inReplyToId: inReplyTo,
          ),
          args
        );
      case TextMessage.type:
      case EmoteMessage.type:
      default:
        final body = content['body'];
        final formattedBody = content['formatted_body'];

        if (msgtype == EmoteMessage.type) {
          return EmoteMessageEvent(
            EmoteMessage(
              body: body,
              formattedBody: formattedBody,
              inReplyToId: inReplyTo,
            ),
            args
          );
        } else {
          return TextMessageEvent(
            TextMessage(
              body: body,
              formattedBody: formattedBody,
              inReplyToId: inReplyTo,
            ),
            args
          );
        }

    }
  }
}

class TextMessage extends MessageEventContent {
  static const String type = 'm.text';
  @override
  final String messageType = type;

  final String body;
  final String formattedBody;

  @override
  final EventId inReplyToId;

  TextMessage({@required this.body, String formattedBody, this.inReplyToId})
    : formattedBody = formattedBody ?? body;

  @override
  Map<String, dynamic> toJson() => super.toJson()
    ..addAll({
      'body': body,
      'formatted_body': formattedBody
    });
}

class TextMessageEvent extends MessageEvent {

  @override
  final TextMessage content;

  TextMessageEvent(
    this.content,
    RoomEventArgs args
  ) : super(content, args);
}

class EmoteMessage extends TextMessage {
  static const String type = 'm.emote';
  @override
  final String messageType = type;

  EmoteMessage({
    @required String body,
    String formattedBody,
    EventId inReplyToId
  }) : super(
    body: body,
    formattedBody: formattedBody,
    inReplyToId: inReplyToId
  );
}

class EmoteMessageEvent extends TextMessageEvent {

  @override
  final EmoteMessage content;

  EmoteMessageEvent(
    this.content,
    RoomEventArgs args
  ) : super(content, args);
}

class ImageMessage extends MessageEventContent {
  static const String type = 'm.image';
  @override
  final String messageType = type;

  final String body;
  final Uri url;
  final ImageInfo info;

  @override
  final EventId inReplyToId;

  ImageMessage({@required this.body, this.url, this.info, this.inReplyToId});

  @override
  Map<String, dynamic> toJson() => super.toJson()
    ..addAll({
      'body': body,
      'url': url.toString(),
      'info': {
        'w': info.width,
        'h': info.height
      }
    });
}

class ImageInfo {
  final int width;
  final int height;

  ImageInfo({@required this.width, @required this.height});
}

class ImageMessageEvent extends MessageEvent {

  @override
  final ImageMessage content;

  ImageMessageEvent(
      this.content,
      RoomEventArgs args)
      : super(content, args);
}

==> matrix_sdk-0.11.0.tar.gz: lib/src/event/room/message_event.dart <==
// Copyright (C) 2019  Wilko Manger
// Copyright (C) 2019  Mathieu Velten
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'package:matrix_sdk/src/event/event.dart';
import 'package:matrix_sdk/src/event/room/room_event.dart';
import 'package:matrix_sdk/src/identifier.dart';
import 'package:meta/meta.dart';

const String _messageEventType = 'm.room.message';

abstract class MessageEventContent extends EventContent {

  @override
  final String matrixType = _messageEventType;

  @protected
  String get messageType;

  EventId get inReplyToId;

  @override
  Map<String, dynamic> toJson() {
    final Map<String, dynamic> json = { 'msgtype': messageType };

    if (inReplyToId != null) {
      json.addAll({
        'm.relates_to': {
          'm.in_reply_to': {
            'event_id': inReplyToId.toString()
          }
        }
      });
    }

    return json;
  }
}

abstract class MessageEvent extends RoomEvent {

  static const String type = _messageEventType;

  @override
  final String matrixType = _messageEventType;

  @override
  final MessageEventContent content;

  MessageEvent(
    this.content,
    RoomEventArgs args)
    : super(args);

  static MessageEvent fromJson(RoomEventArgs args,
                                Map<String, dynamic> json) {

    final content = json['content'] as Map<String, dynamic>;
    final msgtype = content['msgtype'];

    var inReplyTo;
    if (content.containsKey('m.relates_to')
      && content['m.relates_to']?.containsKey('m.in_reply_to') == true) {
      final repliesTo = content['m.relates_to']['m.in_reply_to']['event_id'];
      inReplyTo = EventId(repliesTo);
    }

    switch (msgtype) {
      case ImageMessage.type:
        final body = content['body'];

        var url = content['url'];
        if (url == null) {
          return null;
        }

        var info = content['info'];
        if (info != null) {
          var width = content['info']['w'];
          var height = content['info']['h'];

          if (width is! int) {
            width = width.round();
          }

          if (height is! int) {
            height = height.round();
          }

          info = ImageInfo(
            width: width,
            height: height
          );
        }

        url = Uri.parse(url);
        return ImageMessageEvent(
          ImageMessage(
            body: body,
            url: url,
            info: info,
            inReplyToId: inReplyTo,
          ),
          args
        );
      case AudioMessage.type:
        final body = content['body'];

        var url = content['url'];
        if (url == null) {
          return null;
        }

        var info = content['info'];
        if (info != null) {
          info = AudioInfo(
            duration: content['info']['duration'],
            mimetype: content['info']['mimetype'],
            size: content['info']['size']
          );
        }

        url = Uri.parse(url);
        return AudioMessageEvent(
          AudioMessage(
            body: body,
            url: url,
            info: info,
            inReplyToId: inReplyTo,
          ),
          args
        );
      case TextMessage.type:
      case EmoteMessage.type:
      default:
        final body = content['body'];
        final formattedBody = content['formatted_body'];

        if (msgtype == EmoteMessage.type) {
          return EmoteMessageEvent(
            EmoteMessage(
              body: body,
              formattedBody: formattedBody,
              inReplyToId: inReplyTo,
            ),
            args
          );
        } else {
          return TextMessageEvent(
            TextMessage(
              body: body,
              formattedBody: formattedBody,
              inReplyToId: inReplyTo,
            ),
            args
          );
        }

    }
  }
}

class TextMessage extends MessageEventContent {
  static const String type = 'm.text';
  @override
  final String messageType = type;

  final String body;
  final String formattedBody;

  @override
  final EventId inReplyToId;

  TextMessage({@required this.body, String formattedBody, this.inReplyToId})
    : formattedBody = formattedBody ?? body;

  @override
  Map<String, dynamic> toJson() => super.toJson()
    ..addAll({
      'body': body,
      'formatted_body': formattedBody
    });
}

class TextMessageEvent extends MessageEvent {

  @override
  final TextMessage content;

  TextMessageEvent(
    this.content,
    RoomEventArgs args
  ) : super(content, args);
}

class EmoteMessage extends TextMessage {
  static const String type = 'm.emote';
  @override
  final String messageType = type;

  EmoteMessage({
    @required String body,
    String formattedBody,
    EventId inReplyToId
  }) : super(
    body: body,
    formattedBody: formattedBody,
    inReplyToId: inReplyToId
  );
}

class EmoteMessageEvent extends TextMessageEvent {

  @override
  final EmoteMessage content;

  EmoteMessageEvent(
    this.content,
    RoomEventArgs args
  ) : super(content, args);
}

class ImageMessage extends MessageEventContent {
  static const String type = 'm.image';
  @override
  final String messageType = type;

  final String body;
  final Uri url;
  final ImageInfo info;

  @override
  final EventId inReplyToId;

  ImageMessage({@required this.body, this.url, this.info, this.inReplyToId});

  @override
  Map<String, dynamic> toJson() => super.toJson()
    ..addAll({
      'body': body,
      'url': url.toString(),
      'info': {
        'w': info.width,
        'h': info.height
      }
    });
}

class ImageInfo {
  final int width;
  final int height;

  ImageInfo({this.width, this.height});
}

class ImageMessageEvent extends MessageEvent {

  @override
  final ImageMessage content;

  ImageMessageEvent(
      this.content,
      RoomEventArgs args)
      : super(content, args);
}

class AudioMessage extends MessageEventContent {
  static const String type = 'm.audio';
  @override
  final String messageType = type;

  final String body;
  final Uri url;
  final AudioInfo info;

  @override
  final EventId inReplyToId;

  AudioMessage({@required this.body, this.url, this.info, this.inReplyToId});

  @override
  Map<String, dynamic> toJson() {
    final json = <String, dynamic>{
      'body': body,
      'url': url.toString(),
    };

    if (info != null) {
      json['info'] = {
        'duration': info.duration,
        'mimetype': info.mimetype,
        'size': info.size
      };
    }
    return super.toJson()..addAll(json);
  }
}

class AudioInfo {
  final int duration;
  final String mimetype;
  final int size;

  AudioInfo({this.duration, this.mimetype, this.size});
}

class AudioMessageEvent extends MessageEvent {

  @override
  final AudioMessage content;

  AudioMessageEvent(
      this.content,
      RoomEventArgs args)
      : super(content, args);
}
